#===============================================================================
# CERN (BE-CO-HT)
# Register test for CONV-TTL-BLO
#===============================================================================
# author: Theodor Stana (t.stana@cern.ch)
#
# date of creation: 2013-08-13
#
# version: 1.0
#
# description: 
#
# dependencies:
#
# references:
#   [1] M25P32 32Mb 3V NOR Serial Flash Embedded Memory
#       http://cern.ch/go/z7pw
#===============================================================================
# GNU LESSER GENERAL PUBLIC LICENSE
#===============================================================================
# This source file is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version. This source is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details. You should have
# received a copy of the GNU Lesser General Public License along with this
# source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
#===============================================================================
# last changes:
#    2013-08-13    Theodor Stana     t.stana@cern.ch     File created
#===============================================================================
#  TODO: - 
#===============================================================================

import random
import sys
import time
sys.path.append("../vbcp")
from vbcp import *
from vbcpexcept import *

MB_BASE = 0x40
MB_CR_OFS = 0x00
MB_SR_OFS = 0x04
MB_GBBAR_OFS = 0x08
MB_MBBAR_OFS = 0x0C
MB_FAR_OFS   = 0x10

# This function handles the OR-ing together of the data bytes and the control
# byte in FAR.
#
# Formatting of data bytes in the FAR register should be done outside this
# function.
def spi_transfer(nbytes, cs, dat):
  retval = 0
  wval = []

  # control byte, to be shifted left by 24
  ctrl = ((cs << 3) | 0x4) | (nbytes-1)

  # Use appropriate command by type
  #
  # write - we send up to three data bytes in one FAR register write
  # writemregs - we send up to 24 data bytes in eight FAR register writes
  if isinstance(dat,int):
    ctb.write(MB_BASE+MB_FAR_OFS, (ctrl << 24) | dat)
  else:
    for i in xrange(len(dat)):
      wval.append((ctrl << 24) | dat[i])
    ctb.writemregs(MB_BASE+MB_FAR_OFS, wval)

  # Read the data and prepare the return value
  while (retval & (1 << 28) == 0):
    retval = ctb.read(MB_BASE+MB_FAR_OFS)

  return retval & 0xFFFFFF

# This function is used to write data bytes to flash. It calls spi_transfer
# and sends the commands to the flash chip. You can follow the logic of this
# function by looking at the sequence of the write command in [1]
def flash_write(addr, dat):

  # write enable cmd
  spi_transfer(1,1,0x06)
  spi_transfer(1,0,0)

  # write page cmd
  spi_transfer(1,1,0x02)

  # send address in reverse order
  addr = ((addr & 0xff0000) >> 16) | (((addr & 0xff00) >> 8) << 8) | ((addr & 0xff) << 16)
  spi_transfer(3,1,addr)

  # Now, massage the data array into 3-byte commands sent into 32-bit register,
  # in batches of eight registers (writemregs command).
  i = 0
  l = len(dat)
  while (l):
    wval = []
    # If we have more than 24 bytes left in the data array, we can use the full
    # writemregs
    if (int(l/24)):
      for j in xrange(0, 8):
        wval.append((dat[(i+2)+3*j] << 16) | (dat[(i+1)+3*j] << 8) | dat[i+3*j])
      spi_transfer(3, 1, wval)
      i += 24;
      l -= 24;

    # When we're left to fewer than 24 bytes in the array, try to use one more
    # writemregs with as many regs as possible
    elif (int(l/3)):
      for j in xrange(0, l/3):
        wval.append((dat[(i+2)+3*j] << 16) | (dat[(i+1)+3*j] << 8) | dat[i+3*j])
      spi_transfer(3,1,wval);
      i += 3 * int(l/3)
      l -= 3 * int(l/3)

    # When we're down to less than three bytes, just use simple one-byte xfers
    elif (l):
      for j in xrange(0, l):
        spi_transfer(1,1,dat[i])
        i += 1
        l -= 1

  spi_transfer(1,0,0)


# This function is used to read a number of data bytes from the flash memory
# It returns the values of the three consecutive flash registers packed into
# one 24-bit data value in little-endian order
def flash_read(addr, nrbytes):
  ret = []
  spi_transfer(1,1,0x0b)

  # send address in reverse order
  addr = ((addr & 0xff0000) >> 16) | (((addr & 0xff00) >> 8) << 8) | ((addr & 0xff) << 16)
  spi_transfer(3,1,addr)
  spi_transfer(1,1,0)

  # Read bytes in groups of three
  for i in range(nrbytes):
    ret.append(spi_transfer(1,1,0))
  spi_transfer(1,0,0)

  return ret

# Send a sector erase command
def flash_serase(addr):
  spi_transfer(1,1,0x06)
  spi_transfer(1,0,0)
  spi_transfer(1,1,0xD8)
  # send address in reverse order
  addr = ((addr & 0xff0000) >> 16) | (((addr & 0xff00) >> 8) << 8) | ((addr & 0xff) << 16)
  spi_transfer(3,1,addr)
  spi_transfer(1,0,0)

# Send a block erase command
def flash_berase():
  spi_transfer(1,1,0x06)
  spi_transfer(1,0,0)
  spi_transfer(1,1,0xc7)
  spi_transfer(1,0,0)

# Read status register command
def flash_rsr():
  spi_transfer(1,1,0x05)
  ret = spi_transfer(1,1,0)
  spi_transfer(1,0,0)
  return ret


#=============================================================================#


#
# Read from flash
#
def _read(addr):
  print "Reading 256 bytes from 0x%06X" % addr
  tr0 = time.time()
  rd = flash_read(addr,256)
  tr1 = time.time()
  rd = "".join(["%02X" % b for b in rd])
  print rd

#
# Write to flash
#
def _write(addr):

  # Ookay, now we start to write the flash
  print "Writing bitstream..."

  # Prepare time arrays, to calculate average time each operation takes
  tdat = []
  twr  = []
  twa  = []
  te = []
  tp = []

  # Ask for and open bitstream file
  fname = raw_input("Input file name: ")
  f = open(fname,'r')

  tw0 = time.time()

  # Each line in the input file contains the data for one page
  for fdata in f:
    print addr
    data = []

    # Erase on sector boundary
    if not (addr % 0x10000):
      print 'erase'
      te1 = time.time()
      flash_serase(addr)
      while (flash_rsr() & 0x01):
        pass
      te2 = time.time()
      te.append(te2-te1)

    # Read the hex values from the input file, which contains page data as 
    # strings of hex values, one hex value (i.e., one character) per nibble
    try:
      t1 = time.time()
      for i in xrange(256):
        data.append(int(fdata[0:2],16))
        fdata = fdata[2:]
      t2 = time.time()

    # The last page of the flash will not be filled up with bytes, so
    # data.append() above will throw a ValueError; catch this exception
    # and use it to mark the end
    except ValueError:
      print "reached end"
      break

    # The finally block gets executed no matter what, so we use it to actually
    # write data to the flash, one page at a time. After every write, the
    # address is incremented to the next page address.
    finally:
      t3 = time.time()
      flash_write(addr, data)
      t4 = time.time()
      while (flash_rsr() & 0x01):
        pass
      t5 = time.time()
      addr += 256
      tdat.append(t2-t1)
      twr.append(t4-t3)
      twa.append(t5-t4)
      tp.append(t5-t3)

      #if (addr == 0x10000):
      #  break

      #print data
      #print len(data)

  tw1 = time.time()

  # Close file handle
  f.close()

  # Phew! We're ready, so calculate the mean time it took for each process
  print "DONE!"
  print "data time:  %2.6f" % float(sum(tdat)/len(tdat))
  print "erase time: %2.6f" % float(sum(te)/len(te))
  print "write time: %2.6f" % float(sum(twr)/len(twr)) #(t4-t3)
  print "wait time:  %2.6f" % float(sum(twa)/len(twa)) #(t5-t4)
  print "page write time: %2.6f" % float(sum(tp)/len(tp))
  print "total write time: %2.6f" % float(tw1-tw0)

#
# Start IPROG sequence
#
def _iprog(addr):
  print "Issuing IPROG command..."
  ctb.write(MB_BASE+MB_GBBAR_OFS, 0x44 | (0x0b << 24))
  ctb.write(MB_BASE+MB_MBBAR_OFS, addr | (0x0b << 24))
  ctb.write(MB_BASE+MB_CR_OFS, 0x10000)
  try:
    ctb.write(MB_BASE+MB_CR_OFS, 0x20000)
  except NAckError:
    pass

  # Set timeout to be sixty seconds from now...
  t0 = time.time()
  t1 = t0 + 20

  # and wait for the FPGA to gracefully respond, or die trying
  while (1):
    try:
      if (time.time() >= t1):
        print "Timeout, IPROG unsuccessful!"
        break
      if ((ctb.read(0x4) & 0xFFFF) > 0x0200):
        print "IPROG successful!"
        break
    except NAckError:
        continue

def _rdcfgreg():
  print "Press 'q' to end config reg readout"
  while 1:
    try:
      reg = raw_input('Address (hex): ')
      reg = int(reg, 16)
      if (reg < 0x00) or (reg > 0x22):
        raise ValueError
      ctb.write(MB_BASE+MB_CR_OFS, (1 << 6) | reg)
      val = ctb.read(MB_BASE+MB_SR_OFS)
      if (val & (1 << 16)):
        print "REG(0x%02X) = 0x%04X" % (reg, val & 0xFFFF)
      else:
        print "Status invalid!"
    except ValueError:
      if (reg == 'q'):
        break
      print "Please input a hex value in the range [0x00, 0x22] or 'q' to quit"



#=============================================================================#



# MAIN "FUNCTION"
if __name__ == "__main__":
  ip   = "192.168.20.22"  # raw_input("Crate IP? ")
  user = "admin" # raw_input("Username? ")
  pwd  = "ADMIN" # raw_input("Password? ")

  # Wait for proper slot number
  while 1:
    try:
      slot = raw_input("Slot no.? ")
      slot = int(slot)
      ctb  = VBCP(ip, user, pwd, slot)
      break
    except TypeError as e:
      print "Please input a decimal slot number."
    except SlotError as e:
      print e.strerror
    except KeyboardInterrupt:
      sys.exit();
    except:
      print "Unexpected error: ", sys.exc_info()[0]

  # Open VBCP connection
  ctb.open_conn()

  # Set MB_BASE to value in pulsetest bitstream, if this bitstream is present
  if (ctb.read(0x04) & 0xFFFF == 0x9999):
    MB_BASE = 0x300;

  # Read config reg, if user wants to
  while 1:
    rdcfgreg = raw_input("Read config reg? (y/n) ")
    if (rdcfgreg == 'y'):
      _rdcfgreg()
      break
    elif (rdcfgreg == 'n'):
      break

  # Ask else to do (read/write/iprog)
  while 1:
    rd = raw_input("Read first page? (y/n) ")
    if (rd == 'y') or (rd == 'n'):
      break

  while 1:
    wr = raw_input("Write to flash?  (y/n) ")
    if (wr == 'y'):
      while 1:
        wr = raw_input("Are you sure?    (y/n) ")
        if (wr == 'y') or (wr == 'n'):
          break
      break
    elif (wr == 'n'):
      break

  while 1:
    iprog = raw_input("Issue IPROG?     (y/n) ")
    if (iprog == 'y') or (iprog == 'n'):
      break
  
  # Ask for multiboot address
  while 1:
    try:
      multiboot_addr = raw_input("MultiBoot bitstream address (hex): ")
      multiboot_addr = int(multiboot_addr, 16)
      print '0x%06x' % multiboot_addr
      break
    except ValueError as e:
      print "Please input a hexadecimal address!"
    except KeyboardInterrupt:
      sys.exit()
    except:
      print "Unexpected error: ", sys.exc_info()[0]


  # Now do something based on user selection

  # read from flash
  if (rd == 'y'):
    _read(multiboot_addr)

  # write to flash
  if (wr == 'y'):
    _write(multiboot_addr)

  # Finally, issue the reprogramming (IPROG) command
  if (iprog == 'y'):
    _iprog(multiboot_addr)

  # Close VBCP connection
  ctb.close_conn()

