#===============================================================================
# CERN (BE-CO-HT)
# Register test for CONV-TTL-BLO
#===============================================================================
# author: Theodor Stana (t.stana@cern.ch)
#
# date of creation: 2013-08-13
#
# version: 1.0
#
# description: 
#
# dependencies:
#
# references:
#===============================================================================
# GNU LESSER GENERAL PUBLIC LICENSE
#===============================================================================
# This source file is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version. This source is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details. You should have
# received a copy of the GNU Lesser General Public License along with this
# source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
#===============================================================================
# last changes:
#    2013-08-13    Theodor Stana     t.stana@cern.ch     File created
#===============================================================================
#  TODO: - 
#===============================================================================

import random
import sys
import time
sys.path.append("../vbcp")
from vbcp import *

RAMSTART = 0x100
RAMEND   = 0x900

if __name__ == "__main__":
  ip   = "elma"  # raw_input("Crate IP? ")
  user = "admin" # raw_input("Username? ")
  pwd  = "ADMIN" # raw_input("Password? ")

  # Wait for proper slot number
  while 1:
    try:
      slot = raw_input("Slot no.? ")
      slot = int(slot)
      ctb  = VBCP(ip, user, pwd, slot)
      break
    except TypeError as e:
      print "Please input a decimal slot number."
    except SlotError as e:
      print e.strerror
    except KeyboardInterrupt:
      sys.exit()
    except:
      print "Unexpected error: ", sys.exc_info()[0]

  # Ask how long to run the test
  print "How long would you like to run the test?"
  while 1:
    try:
      hrs = raw_input("  hrs: ")
      hrs = int(hrs)
      if (hrs < 0):
        raise Exception
      break
    except KeyboardInterrupt:
      sys.exit()
    except:
      print "    Hours must be a decimal number >= 0!"

  while 1:
    try:
      mins = raw_input("  mins: ")
      mins = int(mins)
      if (mins < 0):
        raise Exception
      break
    except KeyboardInterrupt:
      sys.exit()
    except:
      print "    Minutes must be a decimal number >= 0!"

  while 1:
    try:
      secs = raw_input("  secs: ")
      secs = int(secs)
      if (secs < 0):
        raise Exception
      break
    except KeyboardInterrupt:
      sys.exit()
    except:
      print "    Seconds must be a decimal number >= 0!"

  # Compute test end time according to current time
  # 
  # time.time() returns time since start of epoch
  # in seconds, so add secs, mins and hrs to that
  end_time  = time.time() + secs + 60*mins + 3600*hrs

  # Generate output file named according to current time
  fname = time.strftime("c-%Y-%m-%d-%Hh%M", time.localtime()) + ".txt"
  f     = open(fname, "w")

  # Write current time and start of test to output file
  f.write("------------------------------------------------------------\r\n")
  f.write("%s: starting test\r\n" % time.strftime("%Y-%m-%d-%Hh%Mm%Ss", time.localtime())) 
  print "%s: starting test" % time.strftime("%Y-%m-%d-%Hh%Mm%Ss", time.localtime())
  f.write("------------------------------------------------------------\r\n")

  # Open VBCP connection
  ctb.open_conn()

  # Init counters
  wexcep_cnt   = 0
  rexcep_cnt   = 0
  mismatch_cnt = 0
  addr = RAMSTART

  # Run test loop for the specified time
  while time.time() < end_time:

    # Increment regaddr, reset on overflow above 2^12
    addr += 4
    if (addr > RAMEND):
      addr = RAMSTART

    # Write the regaddress value to the register
    wval = addr
    
    #f.write("a:%x w:%x " % (addr, wval))

    # Try a write to CONV-TTL-BLO, exception if unsuccessful
    try:
      ctb.write(addr, wval)
    except VBCPException as e:
      f.write("wex / %s / %s\r\n" % (time.strftime("%Y-%m-%d-%Hh%Mm%Ss",time.localtime()), e.strerror))
      wexcep_cnt += 1

    # Try a read from CONV-TTL-BLO, exception if unsuccessful
    try:
      rval = ctb.read(addr)
      #f.write("r:%x\r\n" % rval)

      # Check if read value is same as written value
      if rval != wval:
        mismatch_cnt += 1

    except VBCPException as e:
      f.write("rex / %s / %s\r\n" % (time.strftime("%Y-%m-%d-%Hh%Mm%Ss",time.localtime()), e.strerror))
      rexcep_cnt += 1

  # Close VBCP connection and print final values
  ctb.close_conn()

  f.write("------------------------------------------------------------\r\n")
  f.write("%s: ending test\r\n" % time.strftime("%Y-%m-%d-%Hh%Mm%Ss", time.localtime()))
  print "%s: ending test" % time.strftime("%Y-%m-%d-%Hh%Mm%Ss", time.localtime())
  f.write("------------------------------------------------------------\r\n")
  f.write("Write transactions: %d\r\n" % ctb.write_cnt)
  f.write("Write exceptions: %d\r\n\r\n" % wexcep_cnt)
  f.write("Read transactions: %d\r\n" % ctb.read_cnt)
  f.write("Read exceptions:  %d\r\n\r\n" % rexcep_cnt)
  f.write("R/W mismatches:   %d\r\n" % mismatch_cnt)
  f.write("------------------------------------------------------------\r\n")

  # Close output file
  f.close()

  # Print output file name
  print
  print "Test results can be found in " + fname
