#===============================================================================
# CERN (BE-CO-HT)
# VME Board Control Protocol (VBCP) class
#===============================================================================
# author: Theodor Stana (t.stana@cern.ch)
#
# date of creation: 2013-08-13
#
# version: 1.0
#
# description: 
#
# dependencies:
#
# references:
#===============================================================================
# GNU LESSER GENERAL PUBLIC LICENSE
#===============================================================================
# This source file is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version. This source is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details. You should have
# received a copy of the GNU Lesser General Public License along with this
# source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
#===============================================================================
# last changes:
#    2013-08-13    Theodor Stana     t.stana@cern.ch     File created
#    2013-10-14    Theodor Stana     t.stana@cern.ch     Updated VBCP protocol
#                                                        with hex addresses
#                                                        instead of reg numbers
#===============================================================================
#  TODO: - 
#===============================================================================

import socket
from vbcpexcept import *

class VBCP:

  """Access a VME crate through Telnet."""

  def __init__(self, ip, user, pwd, slot):

    """Initialize VBCP class.

    The arguments to init the class are mandatory:
    ip   -- IP address of VME crate
    user -- username to login to VME crate
    pwd  -- password to login to VME crate
    slot -- slot number of VME board
    """
    
    self.ip   = ip
    self.user = user
    self.pwd  = pwd
    if (slot <= 21):
      self.slot = slot
    else:
      raise SlotError(slot)
    
    self.write_cnt = 0
    self.read_cnt  = 0

  def open_conn(self):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM);
    s.connect((self.ip, 23))
    s.recv(30)
    s.send(self.user + "\r\n")
    s.recv(30)
    s.send(self.pwd + "\r\n")
    s.recv(30)
    self.handle = s

  def close_conn(self):
    self.handle.close()

  def writemregs(self, addr, val):
    self.write_cnt += 1
    cmd = "writemregs %d %x %s\r\n" % (self.slot, addr, ' '.join(format(b,'x') for b in val))
    #print cmd
    self.handle.send(cmd)
    _strip_resp(self.handle.recv(30))

  def readmregs(self, addr, nrregs):
    self.read_cnt += 1
    #addr = addr/4 + 1
    cmd = "readmregs %d %x %d\r\n" % (self.slot, addr, nrregs)
    self.handle.send(cmd)
    resp = _strip_resp(self.handle.recv(30))
    return resp

  def write(self, addr, val):
    self.write_cnt += 1
    #addr = addr/4 + 1
    cmd = "writereg %d %x %x\r\n" % (self.slot, addr, val)
    #print cmd
    self.handle.send(cmd)
    _strip_resp(self.handle.recv(30))

  def read(self, addr):
    self.read_cnt += 1
    #addr = addr/4 + 1
    cmd = "readreg %d %x\r\n" % (self.slot, addr)
    #print cmd
    self.handle.send(cmd)
    resp = _strip_resp(self.handle.recv(30))
    return resp

def _strip_resp(msg):

    """Strip useful message from SysMon response.
    
    The response string from the SysMon is stripped of unnecessary characters
    and a "useful" string is returned. In case one of the error messages appear,
    the function raises one of the VBCP exceptions."""

    #print msg.splitlines()
    msg = msg.splitlines()[1]

    if "Not Ack" in msg:
      raise NAckError

    if "Invalid register number!" in msg:
      raise RegNumberError

    if "Bad command!" in msg:
      raise InvalidCmdError

    if "Read Data:" in msg:
      msg = msg.split(" ")[3]
      msg = int(msg,16)

    return msg
