#===============================================================================
# CERN (BE-CO-HT)
# VBCP exceptions
#===============================================================================
# author: Theodor Stana (t.stana@cern.ch)
#
# date of creation: 2013-08-14
#
# version: 1.0
#
# description: 
#
# dependencies:
#
# references:
#===============================================================================
# GNU LESSER GENERAL PUBLIC LICENSE
#===============================================================================
# This source file is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version. This source is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details. You should have
# received a copy of the GNU Lesser General Public License along with this
# source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
#===============================================================================
# last changes:
#    2013-08-14    Theodor Stana     t.stana@cern.ch     File created
#===============================================================================
#  TODO: - 
#===============================================================================

#===============================================================================
# Main exception class
#===============================================================================
class VBCPException(Exception):

  """Throw VBCP exceptions.

  Subclasses:
  * SlotError
  * NAckError
  """

  pass

#===============================================================================
# Slot value exception
#===============================================================================
class SlotError(VBCPException):

  """Throw a slot exception if slot number greater than 21."""

  def __init__(self, value):
    self.value     = value
    self.strerror  = "Input slot number (%d) is greater than 21" % value

  def __str__(self):
    return self.strerror

#===============================================================================
# Not Acknowledged exception
#===============================================================================
class NAckError(VBCPException):

  """Throw NACK exception when the module does not acknowledge."""

  strerror = "VBCP NACK received."

  def __str__(self):
    return self.strerror

#===============================================================================
# Register number exception
#===============================================================================
class RegNumberError(VBCPException):

  """Throw register number exception."""

  strerror = "Invalid register number."

  def __str__(self):
    return self.strerror

#===============================================================================
# Invalid command exception
#===============================================================================
class InvalidCmdError(VBCPException):

  """Throw invalid command exception."""

  strerror = "Bad command exception."

  def __str__(self):
    return self.strerror

